@@ -38,6 +38,7 @@
 #include "backends/ivvassbackend.h"
 #include "backends/navitiabackend.h"
 #include "backends/oebbbackend.h"
+#include "backends/openjourneyplannerbackend.h"
 #include "backends/opentripplannergraphqlbackend.h"
 #include "backends/opentripplannerrestbackend.h"
 #include "gbfs/gbfsbackend.h"
@@ -85,6 +86,8 @@ public:
 
     void readCachedAttributions();
 
+    int queryLocationOnBackend(const LocationRequest &req, LocationReply *reply, const Backend &backend);
+
     Manager *q = nullptr;
     QNetworkAccessManager *m_nam = nullptr;
     std::vector<Backend> m_backends;
@@ -170,6 +173,7 @@ std::unique_ptr<AbstractBackend> ManagerPrivate::loadNetwork(const QJsonObject &
         HafasQueryBackend,
         EfaBackend,
         IvvAssBackend,
+        OpenJourneyPlannerBackend,
         GBFSBackend,
         AccessibilityCloudBackend
     >(type, obj);
@@ -490,7 +494,7 @@ void Manager::setAllowInsecureBackends(bool insecure)
         return;
     }
     d->m_allowInsecure = insecure;
-    emit configurationChanged();
+    Q_EMIT configurationChanged();
 }
 
 JourneyReply* Manager::queryJourney(const JourneyRequest &req) const
@@ -499,6 +503,7 @@ JourneyReply* Manager::queryJourney(const JourneyRequest &req) const
     int pendingOps = 0;
 
     // validate input
+    req.validate();
     if (!req.isValid()) {
         reply->addError(Reply::InvalidRequest, {});
         reply->setPendingOps(pendingOps);
@@ -510,22 +515,45 @@ JourneyReply* Manager::queryJourney(const JourneyRequest &req) const
         QSet<QString> triedBackends;
         bool foundNonGlobalCoverage = false;
         for (const auto coverageType : { CoverageArea::Realtime, CoverageArea::Regular, CoverageArea::Any }) {
-            for (const auto &backend: d->m_backends) {
+            const auto checkBackend = [&](const Backend &backend, bool bothLocationMatch) {
                 if (triedBackends.contains(backend.identifier()) || d->shouldSkipBackend(backend, req)) {
-                    continue;
+                    return;
                 }
                 const auto coverage = backend.coverageArea(coverageType);
-                if (coverage.isEmpty() || (!coverage.coversLocation(req.from()) && !coverage.coversLocation(req.to()))) {
-                    continue;
+                if (coverage.isEmpty()) {
+                    return;
                 }
+
+                if (bothLocationMatch) {
+                    if (!coverage.coversLocation(req.from()) || !coverage.coversLocation(req.to())) {
+                        return;
+                    }
+                } else {
+                    if (!coverage.coversLocation(req.from()) && !coverage.coversLocation(req.to())) {
+                        return;
+                    }
+                }
+
                 triedBackends.insert(backend.identifier());
                 foundNonGlobalCoverage |= !coverage.isGlobal();
 
                 if (d->queryJourney(BackendPrivate::impl(backend), req, reply)) {
                     ++pendingOps;
                 }
+            };
+
+            // look for coverage areas which contain both locations first
+            for (const auto &backend: d->m_backends) {
+                checkBackend(backend, true);
+            }
+            if (pendingOps && foundNonGlobalCoverage) {
+                break;
             }
 
+            // if we didn't find one, try with just a single one
+            for (const auto &backend: d->m_backends) {
+                checkBackend(backend, false);
+            }
             if (pendingOps && foundNonGlobalCoverage) {
                 break;
             }
@@ -563,6 +591,9 @@ JourneyReply* Manager::queryJourney(const JourneyRequest &req) const
         }
     }
 
+    if (req.downloadAssets()) {
+        reply->addAttributions(AssetRepository::instance()->attributions());
+    }
     reply->setPendingOps(pendingOps);
     return reply;
 }
@@ -634,6 +665,9 @@ StopoverReply* Manager::queryStopover(const StopoverRequest &req) const
         }
     }
 
+    if (req.downloadAssets()) {
+        reply->addAttributions(AssetRepository::instance()->attributions());
+    }
     reply->setPendingOps(pendingOps);
     return reply;
 }
@@ -643,6 +677,30 @@ StopoverReply* Manager::queryDeparture(const StopoverRequest &req) const
     return queryStopover(req);
 }
 
+int ManagerPrivate::queryLocationOnBackend(const LocationRequest &req, LocationReply *reply, const Backend &backend)
+{
+    auto cache = Cache::lookupLocation(backend.identifier(), req.cacheKey());
+    switch (cache.type) {
+        case CacheHitType::Negative:
+            qCDebug(Log) << "Negative cache hit for backend" << backend.identifier();
+            break;
+        case CacheHitType::Positive:
+            qCDebug(Log) << "Positive cache hit for backend" << backend.identifier();
+            reply->addAttributions(std::move(cache.attributions));
+            reply->addResult(std::move(cache.data));
+            break;
+        case CacheHitType::Miss:
+            qCDebug(Log) << "Cache miss for backend" << backend.identifier();
+            reply->addAttribution(BackendPrivate::impl(backend)->attribution());
+            if (BackendPrivate::impl(backend)->queryLocation(req, reply, nam())) {
+                return 1;
+            }
+            break;
+    }
+
+    return 0;
+}
+
 LocationReply* Manager::queryLocation(const LocationRequest &req) const
 {
     auto reply = d->makeReply<LocationReply>(req);
@@ -657,37 +715,43 @@ LocationReply* Manager::queryLocation(const LocationRequest &req) const
 
     QSet<QString> triedBackends;
     bool foundNonGlobalCoverage = false;
+    const auto loc = req.location();
+    const auto isCountryOnly = !loc.hasCoordinate() && !loc.country().isEmpty() && loc.region().isEmpty();
     for (const auto coverageType : { CoverageArea::Realtime, CoverageArea::Regular, CoverageArea::Any }) {
+        // pass 1: coordinate-based coverage, or nationwide country coverage
         for (const auto &backend : d->m_backends) {
             if (triedBackends.contains(backend.identifier()) || d->shouldSkipBackend(backend, req)) {
                 continue;
             }
             const auto coverage = backend.coverageArea(coverageType);
-            if (coverage.isEmpty() || !coverage.coversLocation(req.location())) {
+            if (coverage.isEmpty() || !coverage.coversLocation(loc)) {
                 continue;
             }
+            if (isCountryOnly && !coverage.hasNationWideCoverage(loc.country())) {
+                continue;
+            }
+
             triedBackends.insert(backend.identifier());
             foundNonGlobalCoverage |= !coverage.isGlobal();
+            pendingOps += d->queryLocationOnBackend(req, reply, backend);
+        }
+        if (pendingOps && foundNonGlobalCoverage) {
+            break;
+        }
 
-            reply->addAttribution(BackendPrivate::impl(backend)->attribution());
-
-            auto cache = Cache::lookupLocation(backend.identifier(), req.cacheKey());
-            switch (cache.type) {
-                case CacheHitType::Negative:
-                    qCDebug(Log) << "Negative cache hit for backend" << backend.identifier();
-                    break;
-                case CacheHitType::Positive:
-                    qCDebug(Log) << "Positive cache hit for backend" << backend.identifier();
-                    reply->addAttributions(std::move(cache.attributions));
-                    reply->addResult(std::move(cache.data));
-                    break;
-                case CacheHitType::Miss:
-                    qCDebug(Log) << "Cache miss for backend" << backend.identifier();
-                    if (BackendPrivate::impl(backend)->queryLocation(req, reply, d->nam())) {
-                        ++pendingOps;
-                    }
-                    break;
+        // pass 2: any country match
+        for (const auto &backend : d->m_backends) {
+            if (triedBackends.contains(backend.identifier()) || d->shouldSkipBackend(backend, req)) {
+                continue;
             }
+            const auto coverage = backend.coverageArea(coverageType);
+            if (coverage.isEmpty() || !coverage.coversLocation(loc)) {
+                continue;
+            }
+
+            triedBackends.insert(backend.identifier());
+            foundNonGlobalCoverage |= !coverage.isGlobal();
+            pendingOps += d->queryLocationOnBackend(req, reply, backend);
         }
         if (pendingOps && foundNonGlobalCoverage) {
             break;
@@ -802,7 +866,7 @@ void Manager::setBackendEnabled(const QString &backendId, bool enabled)
         sortedRemove(d->m_enabledBackends, backendId);
         sortedInsert(d->m_disabledBackends, backendId);
     }
-    emit configurationChanged();
+    Q_EMIT configurationChanged();
 }
 
 QStringList Manager::enabledBackends() const
@@ -831,14 +895,22 @@ void Manager::setDisabledBackends(const QStringList &backendIds)
     }
 }
 
-bool KPublicTransport::Manager::backendsEnabledByDefault() const
+bool Manager::backendsEnabledByDefault() const
 {
     return d->m_backendsEnabledByDefault;
 }
 
-void KPublicTransport::Manager::setBackendsEnabledByDefault(bool byDefault)
+void Manager::setBackendsEnabledByDefault(bool byDefault)
 {
     d->m_backendsEnabledByDefault = byDefault;
 
-    emit configurationChanged();
+    Q_EMIT configurationChanged();
+}
+
+QVariantList Manager::backendsVariant() const
+{
+    QVariantList l;
+    l.reserve(d->m_backends.size());
+    std::transform(d->m_backends.begin(), d->m_backends.end(), std::back_inserter(l), [](const auto &b) { return QVariant::fromValue(b); });
+    return l;
 }
